@@ -27,6 +27,7 @@
 #include <vector>
 
 #include "wabt/cast.h"
+#include "wabt/config.h"
 #include "wabt/common.h"
 #include "wabt/feature.h"
 #include "wabt/opcode.h"
@@ -96,7 +97,7 @@ enum class ObjectKind {
   Last = Instance,
 };
 
-static const int kCommandTypeCount = WABT_ENUM_COUNT(ObjectKind);
+constexpr int kCommandTypeCount = WABT_ENUM_COUNT(ObjectKind);
 
 const char* GetName(Mutability);
 const std::string GetName(ValueType);
@@ -119,7 +120,7 @@ using RefVec = std::vector<Ref>;
 template <typename T, u8 L>
 struct Simd {
   using LaneType = T;
-  static const u8 lanes = L;
+  static constexpr u8 lanes = L;
 
   T v[L];
 
@@ -580,7 +581,7 @@ struct Value {
   };
 
  public:
-#ifndef NDEBUG
+#ifdef WABT_DEBUG
   Value() : v128_(0, 0, 0, 0), type(ValueType::Any) {}
   void SetType(ValueType t) { type = t; }
   void CheckType(ValueType t) const {
@@ -929,7 +930,7 @@ class Global : public Extern {
   Result Get(T* out) const;
   template <typename T>
   Result WABT_VECTORCALL Set(T);
-  Result Set(Store&, Ref);
+  void Set(Store&, Ref);
 
   template <typename T>
   T WABT_VECTORCALL UnsafeGet() const;
@@ -1086,8 +1087,8 @@ enum class RunResult {
 class Thread {
  public:
   struct Options {
-    static const u32 kDefaultValueStackSize = 64 * 1024 / sizeof(Value);
-    static const u32 kDefaultCallStackSize = 64 * 1024 / sizeof(Frame);
+    static constexpr u32 kDefaultValueStackSize = 64 * 1024 / sizeof(Value);
+    static constexpr u32 kDefaultCallStackSize = 64 * 1024 / sizeof(Frame);
 
     u32 value_stack_size = kDefaultValueStackSize;
     u32 call_stack_size = kDefaultCallStackSize;
@@ -1216,6 +1217,12 @@ class Thread {
   template <typename S, typename T>
   RunResult DoSimdDot();
   template <typename S, typename T>
+  RunResult DoSimdDotAdd();
+  template <typename S>
+  RunResult DoSimdRelaxedMadd();
+  template <typename S>
+  RunResult DoSimdRelaxedNmadd();
+  template <typename S, typename T>
   RunResult DoSimdLoadExtend(Instr, Trap::Ptr* out_trap);
   template <typename S, typename T>
   RunResult DoSimdExtaddPairwise();
