@@ -14,21 +14,21 @@
  * limitations under the License.
  */
 
-#include "src/binary-writer-spec.h"
+#include "wabt/binary-writer-spec.h"
 
 #include <cassert>
 #include <cinttypes>
 #include <string_view>
 
-#include "config.h"
+#include "wabt/config.h"
 
-#include "src/binary-writer.h"
-#include "src/binary.h"
-#include "src/cast.h"
-#include "src/filenames.h"
-#include "src/ir.h"
-#include "src/literal.h"
-#include "src/stream.h"
+#include "wabt/binary-writer.h"
+#include "wabt/binary.h"
+#include "wabt/cast.h"
+#include "wabt/filenames.h"
+#include "wabt/ir.h"
+#include "wabt/literal.h"
+#include "wabt/stream.h"
 
 namespace wabt {
 
@@ -125,6 +125,7 @@ void BinaryWriterSpec::WriteEscapedString(std::string_view s) {
 
 void BinaryWriterSpec::WriteCommandType(const Command& command) {
   static const char* s_command_names[] = {
+      "module",
       "module",
       "action",
       "register",
@@ -477,6 +478,25 @@ void BinaryWriterSpec::WriteCommands() {
         break;
       }
 
+      case CommandType::ScriptModule: {
+        auto* script_module_command = cast<ScriptModuleCommand>(command);
+        const auto& module = script_module_command->module;
+        std::string filename = GetModuleFilename(kWasmExtension);
+        WriteLocation(module.loc);
+        WriteSeparator();
+        if (!module.name.empty()) {
+          WriteKey("name");
+          WriteEscapedString(module.name);
+          WriteSeparator();
+        }
+        WriteKey("filename");
+        WriteEscapedString(GetBasename(filename));
+        WriteScriptModule(filename, *script_module_command->script_module);
+        num_modules_++;
+        last_module_index = i;
+        break;
+      }
+
       case CommandType::Action: {
         const Action& action = *cast<ActionCommand>(command)->action;
         WriteLocation(action.loc);
@@ -548,8 +568,17 @@ void BinaryWriterSpec::WriteCommands() {
         WriteSeparator();
         WriteAction(*assert_return_command->action);
         WriteSeparator();
-        WriteKey("expected");
-        WriteConstVector(assert_return_command->expected);
+        const Expectation* expectation = assert_return_command->expected.get();
+        switch (expectation->type()) {
+          case ExpectationType::Values:
+            WriteKey("expected");
+            break;
+
+          case ExpectationType::Either:
+            WriteKey("either");
+            break;
+        }
+        WriteConstVector(expectation->expected);
         break;
       }
 
