@@ -14,16 +14,16 @@
  * limitations under the License.
  */
 
-#include "src/decompiler.h"
+#include "wabt/decompiler.h"
 
-#include "src/decompiler-ast.h"
-#include "src/decompiler-ls.h"
-#include "src/decompiler-naming.h"
+#include "wabt/decompiler-ast.h"
+#include "wabt/decompiler-ls.h"
+#include "wabt/decompiler-naming.h"
 
-#include "src/stream.h"
+#include "wabt/stream.h"
 
 #define WABT_TRACING 0
-#include "src/tracing.h"
+#include "wabt/tracing.h"
 
 #include <inttypes.h>
 
@@ -671,7 +671,7 @@ struct Decompiler {
                          Index index,
                          std::string_view name) {
     // Figure out if this thing is imported, exported, or neither.
-    auto is_import = mc.module.IsImport(kind, Var(index));
+    auto is_import = mc.module.IsImport(kind, Var(index, Location()));
     // TODO: is this the best way to check for export?
     // FIXME: this doesn't work for functions that get renamed in some way,
     // as the export has the original name..
