@@ -14,15 +14,15 @@
  * limitations under the License.
  */
 
-#include "src/resolve-names.h"
+#include "wabt/resolve-names.h"
 
 #include <cassert>
 #include <cstdio>
 
-#include "src/cast.h"
-#include "src/expr-visitor.h"
-#include "src/ir.h"
-#include "src/wast-lexer.h"
+#include "wabt/cast.h"
+#include "wabt/expr-visitor.h"
+#include "wabt/ir.h"
+#include "wabt/wast-lexer.h"
 
 namespace wabt {
 
@@ -77,6 +77,8 @@ class NameResolver : public ExprVisitor::DelegateNop {
   Result EndTryExpr(TryExpr*) override;
   Result OnThrowExpr(ThrowExpr*) override;
   Result OnRethrowExpr(RethrowExpr*) override;
+  Result OnSimdLoadLaneExpr(SimdLoadLaneExpr*) override;
+  Result OnSimdStoreLaneExpr(SimdStoreLaneExpr*) override;
 
  private:
   void PrintError(const Location* loc, const char* fmt, ...);
@@ -470,6 +472,16 @@ Result NameResolver::OnRethrowExpr(RethrowExpr* expr) {
   return Result::Ok;
 }
 
+Result NameResolver::OnSimdLoadLaneExpr(SimdLoadLaneExpr* expr) {
+  ResolveMemoryVar(&expr->memidx);
+  return Result::Ok;
+}
+
+Result NameResolver::OnSimdStoreLaneExpr(SimdStoreLaneExpr* expr) {
+  ResolveMemoryVar(&expr->memidx);
+  return Result::Ok;
+}
+
 void NameResolver::VisitFunc(Func* func) {
   current_func_ = func;
   if (func->decl.has_func_type) {
@@ -577,6 +589,10 @@ void NameResolver::VisitCommand(Command* command) {
       VisitModule(&cast<ModuleCommand>(command)->module);
       break;
 
+    case CommandType::ScriptModule:
+      VisitModule(&cast<ScriptModuleCommand>(command)->module);
+      break;
+
     case CommandType::Action:
     case CommandType::AssertReturn:
     case CommandType::AssertTrap:
