@@ -99,8 +99,8 @@ static void ParseOptions(int argc, char** argv) {
 namespace spectest {
 
 class Command;
-typedef std::unique_ptr<Command> CommandPtr;
-typedef std::vector<CommandPtr> CommandPtrVector;
+using CommandPtr = std::unique_ptr<Command>;
+using CommandPtrVector = std::vector<CommandPtr>;
 
 class Script {
  public:
@@ -155,7 +155,7 @@ class ActionCommandBase : public CommandMixin<TypeEnum> {
   Action action;
 };
 
-typedef ActionCommandBase<CommandType::Action> ActionCommand;
+using ActionCommand = ActionCommandBase<CommandType::Action>;
 
 class RegisterCommand : public CommandMixin<CommandType::Register> {
  public:
@@ -280,6 +280,7 @@ class AssertReturnCommand : public CommandMixin<CommandType::AssertReturn> {
  public:
   Action action;
   std::vector<ExpectedValue> expected;
+  bool expect_either;
 };
 
 template <CommandType TypeEnum>
@@ -289,9 +290,9 @@ class AssertTrapCommandBase : public CommandMixin<TypeEnum> {
   std::string text;
 };
 
-typedef AssertTrapCommandBase<CommandType::AssertTrap> AssertTrapCommand;
-typedef AssertTrapCommandBase<CommandType::AssertExhaustion>
-    AssertExhaustionCommand;
+using AssertTrapCommand = AssertTrapCommandBase<CommandType::AssertTrap>;
+using AssertExhaustionCommand =
+    AssertTrapCommandBase<CommandType::AssertExhaustion>;
 
 template <CommandType TypeEnum>
 class AssertModuleCommand : public CommandMixin<TypeEnum> {
@@ -301,13 +302,13 @@ class AssertModuleCommand : public CommandMixin<TypeEnum> {
   std::string text;
 };
 
-typedef AssertModuleCommand<CommandType::AssertMalformed>
-    AssertMalformedCommand;
-typedef AssertModuleCommand<CommandType::AssertInvalid> AssertInvalidCommand;
-typedef AssertModuleCommand<CommandType::AssertUnlinkable>
-    AssertUnlinkableCommand;
-typedef AssertModuleCommand<CommandType::AssertUninstantiable>
-    AssertUninstantiableCommand;
+using AssertMalformedCommand =
+    AssertModuleCommand<CommandType::AssertMalformed>;
+using AssertInvalidCommand = AssertModuleCommand<CommandType::AssertInvalid>;
+using AssertUnlinkableCommand =
+    AssertModuleCommand<CommandType::AssertUnlinkable>;
+using AssertUninstantiableCommand =
+    AssertModuleCommand<CommandType::AssertUninstantiable>;
 
 class AssertExceptionCommand
     : public CommandMixin<CommandType::AssertException> {
@@ -1078,7 +1079,12 @@ wabt::Result JSONParser::ParseCommand(CommandPtr* out_command) {
     EXPECT(",");
     CHECK_RESULT(ParseAction(&command->action));
     EXPECT(",");
-    EXPECT_KEY("expected");
+    if (Match("\"either\"")) {
+      EXPECT(":");
+      command->expect_either = true;
+    } else {
+      EXPECT_KEY("expected");
+    }
     CHECK_RESULT(ParseExpectedValues(&command->expected));
     *out_command = std::move(command);
   } else if (Match("\"assert_trap\"")) {
@@ -1226,6 +1232,7 @@ CommandRunner::CommandRunner() : store_(s_features) {
   } const print_funcs[] = {
       {"print", interp::FuncType{{}, {}}},
       {"print_i32", interp::FuncType{{ValueType::I32}, {}}},
+      {"print_i64", interp::FuncType{{ValueType::I64}, {}}},
       {"print_f32", interp::FuncType{{ValueType::F32}, {}}},
       {"print_f64", interp::FuncType{{ValueType::F64}, {}}},
       {"print_i32_f32", interp::FuncType{{ValueType::I32, ValueType::F32}, {}}},
@@ -1382,9 +1389,9 @@ wabt::Result CommandRunner::ReadInvalidTextModule(
     const std::string& header) {
   std::vector<uint8_t> file_data;
   wabt::Result result = ReadFile(module_filename, &file_data);
-  std::unique_ptr<WastLexer> lexer = WastLexer::CreateBufferLexer(
-      module_filename, file_data.data(), file_data.size());
   Errors errors;
+  std::unique_ptr<WastLexer> lexer = WastLexer::CreateBufferLexer(
+      module_filename, file_data.data(), file_data.size(), &errors);
   if (Succeeded(result)) {
     std::unique_ptr<wabt::Module> module;
     WastParseOptions options(s_features);
@@ -1745,11 +1752,14 @@ wabt::Result CommandRunner::CheckAssertReturnResult(
 
         if (Failed(CheckAssertReturnResult(command, index, lane_expected,
                                            lane_actual, false))) {
-          PrintError(command->line,
-                     "mismatch in lane %u of result %u of assert_return: "
-                     "expected %s, got %s",
-                     lane, index, ExpectedValueToString(lane_expected).c_str(),
-                     TypedValueToString(lane_actual).c_str());
+          if (print_error) {
+            PrintError(command->line,
+                       "mismatch in lane %u of result %u of assert_return: "
+                       "expected %s, got %s",
+                       lane, index,
+                       ExpectedValueToString(lane_expected).c_str(),
+                       TypedValueToString(lane_actual).c_str());
+          }
           ok = false;
         }
       }
@@ -1790,23 +1800,46 @@ wabt::Result CommandRunner::OnAssertReturnCommand(
     return wabt::Result::Error;
   }
 
-  if (action_result.values.size() != command->expected.size()) {
+  if (command->expect_either) {
+    if (action_result.values.size() != 1) {
+      PrintError(command->line,
+                 "\"either\" requires single result but got %" PRIzd,
+                 action_result.values.size());
+      return wabt::Result::Error;
+    }
+
+    TypedValue actual{action_result.types[0], action_result.values[0]};
+    for (size_t i = 0; i < command->expected.size(); ++i) {
+      const ExpectedValue& expected = command->expected[i];
+      if (Succeeded(
+              CheckAssertReturnResult(command, i, expected, actual, false))) {
+        return wabt::Result::Ok;
+      }
+    }
     PrintError(command->line,
-               "result length mismatch in assert_return: expected %" PRIzd
-               ", got %" PRIzd,
-               command->expected.size(), action_result.values.size());
+               "mismatch in result of assert_return: expected %s (%" PRIzd
+               " alternatives), got %s",
+               ExpectedValueToString(command->expected[0]).c_str(),
+               command->expected.size(), TypedValueToString(actual).c_str());
     return wabt::Result::Error;
+  } else {
+    if (action_result.values.size() != command->expected.size()) {
+      PrintError(command->line,
+                 "result length mismatch in assert_return: expected %" PRIzd
+                 ", got %" PRIzd,
+                 command->expected.size(), action_result.values.size());
+      return wabt::Result::Error;
+    }
+
+    wabt::Result result = wabt::Result::Ok;
+    for (size_t i = 0; i < action_result.values.size(); ++i) {
+      const ExpectedValue& expected = command->expected[i];
+      TypedValue actual{action_result.types[i], action_result.values[i]};
+
+      result |= CheckAssertReturnResult(command, i, expected, actual, true);
+    }
+    return result;
   }
-
-  wabt::Result result = wabt::Result::Ok;
-  for (size_t i = 0; i < action_result.values.size(); ++i) {
-    const ExpectedValue& expected = command->expected[i];
-    TypedValue actual{action_result.types[i], action_result.values[i]};
-
-    result |= CheckAssertReturnResult(command, i, expected, actual, true);
-  }
-
-  return result;
 }
 
 wabt::Result CommandRunner::OnAssertTrapCommand(
