@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 
-#include "src/wat-writer.h"
+#include "wabt/wat-writer.h"
 
 #include <algorithm>
 #include <array>
@@ -27,16 +27,16 @@
 #include <string>
 #include <vector>
 
-#include "src/cast.h"
-#include "src/common.h"
-#include "src/expr-visitor.h"
-#include "src/ir-util.h"
-#include "src/ir.h"
-#include "src/literal.h"
-#include "src/stream.h"
+#include "wabt/cast.h"
+#include "wabt/common.h"
+#include "wabt/expr-visitor.h"
+#include "wabt/ir-util.h"
+#include "wabt/ir.h"
+#include "wabt/literal.h"
+#include "wabt/stream.h"
 
 #define WABT_TRACING 0
-#include "src/tracing.h"
+#include "wabt/tracing.h"
 
 #define INDENT_SIZE 2
 #define NO_FORCE_NEWLINE 0
@@ -1041,6 +1041,7 @@ Result WatWriter::ExprVisitorDelegate::OnSimdLaneOpExpr(SimdLaneOpExpr* expr) {
 Result WatWriter::ExprVisitorDelegate::OnSimdLoadLaneExpr(
     SimdLoadLaneExpr* expr) {
   writer_->WritePutsSpace(expr->opcode.GetName());
+  writer_->WriteMemoryVarUnlessZero(expr->memidx, NextChar::Space);
   if (expr->offset) {
     writer_->Writef("offset=%" PRIaddress, expr->offset);
   }
@@ -1055,6 +1056,7 @@ Result WatWriter::ExprVisitorDelegate::OnSimdLoadLaneExpr(
 Result WatWriter::ExprVisitorDelegate::OnSimdStoreLaneExpr(
     SimdStoreLaneExpr* expr) {
   writer_->WritePutsSpace(expr->opcode.GetName());
+  writer_->WriteMemoryVarUnlessZero(expr->memidx, NextChar::Space);
   if (expr->offset) {
     writer_->Writef("offset=%" PRIaddress, expr->offset);
   }
@@ -1329,7 +1331,7 @@ void WatWriter::WriteBeginFunc(const Func& func) {
     WriteCloseSpace();
   }
 
-  if (module.IsImport(ExternalKind::Func, Var(func_index_))) {
+  if (module.IsImport(ExternalKind::Func, Var(func_index_, Location()))) {
     // Imported functions can be written a few ways:
     //
     //   1. (import "module" "field" (func (type 0)))
