@@ -100,6 +100,24 @@ static void error(const char* file, int line, const char* format, ...) {
     }                                                                    \
   } while (0)
 
+#define ASSERT_RETURN_FUNCREF(f, expected)                                \
+  do {                                                                    \
+    g_tests_run++;                                                        \
+    int trap_code = wasm_rt_impl_try();                                   \
+    if (trap_code) {                                                      \
+      error(__FILE__, __LINE__, #f " trapped (%s).\n",                    \
+            wasm_rt_strerror(trap_code));                                 \
+    } else {                                                              \
+      wasm_rt_funcref_t actual = f;                                       \
+      if (is_equal_wasm_rt_funcref_t(actual, expected)) {                 \
+        g_tests_passed++;                                                 \
+      } else {                                                            \
+        error(__FILE__, __LINE__,                                         \
+              "in " #f ": mismatch between expected and actual funcref"); \
+      }                                                                   \
+    }                                                                     \
+  } while (0)
+
 #define ASSERT_RETURN_NAN_T(type, itype, fmt, f, kind)                        \
   do {                                                                        \
     g_tests_run++;                                                            \
@@ -148,6 +166,8 @@ static void error(const char* file, int line, const char* format, ...) {
 #define ASSERT_RETURN_I64(f, expected) ASSERT_RETURN_T(u64, PRIu64, f, expected)
 #define ASSERT_RETURN_F32(f, expected) ASSERT_RETURN_T(f32, ".9g", f, expected)
 #define ASSERT_RETURN_F64(f, expected) ASSERT_RETURN_T(f64, ".17g", f, expected)
+#define ASSERT_RETURN_EXTERNREF(f, expected) \
+  ASSERT_RETURN_T(wasm_rt_externref_t, "p", f, expected);
 
 #define ASSERT_RETURN_CANONICAL_NAN_F32(f) \
   ASSERT_RETURN_NAN_T(f32, u32, "08x", f, canonical)
@@ -169,6 +189,21 @@ static bool is_equal_u64(u64 x, u64 y) {
 #define is_equal_i32 is_equal_u32
 #define is_equal_i64 is_equal_u64
 
+static bool is_equal_wasm_rt_externref_t(wasm_rt_externref_t x,
+                                         wasm_rt_externref_t y) {
+  return x == y;
+}
+
+static bool is_equal_wasm_rt_funcref_t(wasm_rt_funcref_t x,
+                                       wasm_rt_funcref_t y) {
+  return (x.func_type == y.func_type) && (x.func == y.func) &&
+         (x.module_instance == y.module_instance);
+}
+
+wasm_rt_externref_t spectest_make_externref(u64 x) {
+  return (wasm_rt_externref_t)(x + 1);  // externref(0) is not null
+}
+
 static bool is_equal_f32(f32 x, f32 y) {
   u32 ux, uy;
   memcpy(&ux, &x, sizeof(ux));
@@ -214,7 +249,7 @@ static bool is_arithmetic_nan_f64(u64 x) {
 }
 
 typedef struct Z_spectest_instance_t {
-  wasm_rt_table_t spectest_table;
+  wasm_rt_funcref_table_t spectest_table;
   wasm_rt_memory_t spectest_memory;
   uint32_t spectest_global_i32;
   uint64_t spectest_global_i64;
@@ -235,6 +270,10 @@ void Z_spectestZ_print_i32(Z_spectest_instance_t* instance, uint32_t i) {
   printf("spectest.print_i32(%d)\n", i);
 }
 
+void Z_spectestZ_print_i64(Z_spectest_instance_t* instance, uint64_t i) {
+  printf("spectest.print_i64(%" PRIu64 ")\n", i);
+}
+
 void Z_spectestZ_print_f32(Z_spectest_instance_t* instance, float f) {
   printf("spectest.print_f32(%g)\n", f);
 }
@@ -255,7 +294,7 @@ void Z_spectestZ_print_f64_f64(Z_spectest_instance_t* instance,
   printf("spectest.print_f64_f64(%g %g)\n", d1, d2);
 }
 
-wasm_rt_table_t* Z_spectestZ_table(Z_spectest_instance_t* instance) {
+wasm_rt_funcref_table_t* Z_spectestZ_table(Z_spectest_instance_t* instance) {
   return &instance->spectest_table;
 }
 
@@ -283,7 +322,7 @@ static void init_spectest_module(Z_spectest_instance_t* instance) {
   instance->spectest_global_i32 = 666;
   instance->spectest_global_i64 = 666l;
   wasm_rt_allocate_memory(&instance->spectest_memory, 1, 2);
-  wasm_rt_allocate_table(&instance->spectest_table, 10, 20);
+  wasm_rt_allocate_funcref_table(&instance->spectest_table, 10, 20);
 }
 
 int main(int argc, char** argv) {
