@@ -5,7 +5,10 @@
 
 int main(int argc, char** argv) {
   /* Make sure there is at least one command-line argument. */
-  if (argc < 2) return 1;
+  if (argc < 2) {
+    printf("Invalid argument. Expected '%s NUMBER'\n", argv[0]);
+    return 1;
+  }
 
   /* Convert the argument from a string to an int. We'll implicitly cast the int
   to a `u32`, which is what `fac` expects. */
