@@ -19,6 +19,10 @@
 
 #include "wasm-rt.h"
 
+#ifdef _WIN32
+#include <windows.h>
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -26,12 +30,19 @@ extern "C" {
 /** A setjmp buffer used for handling traps. */
 extern jmp_buf wasm_rt_jmp_buf;
 
-#if WASM_RT_MEMCHECK_SIGNAL_HANDLER_POSIX
+#if WASM_RT_MEMCHECK_SIGNAL_HANDLER && !defined(_WIN32)
 #define WASM_RT_LONGJMP(buf, val) siglongjmp(buf, val)
 #else
 #define WASM_RT_LONGJMP(buf, val) longjmp(buf, val)
+#endif
+
+#if WASM_RT_USE_STACK_DEPTH_COUNT
 /** Saved call stack depth that will be restored in case a trap occurs. */
 extern uint32_t wasm_rt_saved_call_stack_depth;
+#define WASM_RT_SAVE_STACK_DEPTH() \
+  wasm_rt_saved_call_stack_depth = wasm_rt_call_stack_depth
+#else
+#define WASM_RT_SAVE_STACK_DEPTH() (void)0
 #endif
 
 /**
@@ -50,15 +61,9 @@ extern uint32_t wasm_rt_saved_call_stack_depth;
  *   my_wasm_func();
  * ```
  */
-#if WASM_RT_MEMCHECK_SIGNAL_HANDLER_POSIX
-#define wasm_rt_impl_try() \
-  (wasm_rt_set_unwind_target(&wasm_rt_jmp_buf), WASM_RT_SETJMP(wasm_rt_jmp_buf))
-#else
-#define wasm_rt_impl_try()                                    \
-  (wasm_rt_saved_call_stack_depth = wasm_rt_call_stack_depth, \
-   wasm_rt_set_unwind_target(&wasm_rt_jmp_buf),               \
+#define wasm_rt_impl_try()                                                  \
+  (WASM_RT_SAVE_STACK_DEPTH(), wasm_rt_set_unwind_target(&wasm_rt_jmp_buf), \
    WASM_RT_SETJMP(wasm_rt_jmp_buf))
-#endif
 
 #ifdef __cplusplus
 }
