@@ -20,6 +20,7 @@
 #include <setjmp.h>
 #include <stdbool.h>
 #include <stdint.h>
+#include <stdlib.h>
 #include <string.h>
 
 #ifdef __cplusplus
@@ -44,6 +45,12 @@ extern "C" {
 #define wasm_rt_memcpy memcpy
 #endif
 
+#if __has_builtin(__builtin_unreachable)
+#define wasm_rt_unreachable __builtin_unreachable
+#else
+#define wasm_rt_unreachable abort
+#endif
+
 /**
  * Enable memory checking via a signal handler via the following definition:
  *
@@ -52,35 +59,53 @@ extern "C" {
  * This is usually 10%-25% faster, but requires OS-specific support.
  */
 
-/** Check whether the signal handler is supported at all. */
-#if (defined(__linux__) || defined(__unix__) || defined(__APPLE__)) && \
-    defined(__WORDSIZE) && __WORDSIZE == 64
+#ifndef WASM_RT_SKIP_SIGNAL_RECOVERY
+#define WASM_RT_SKIP_SIGNAL_RECOVERY 0
+#endif
 
-/* If the signal handler is supported, then use it by default. */
+/** Signal handler not supported on 32-bit platforms. */
+#if UINTPTR_MAX > 0xffffffff
+
+#define WASM_RT_SIGNAL_RECOVERY_SUPPORTED 1
+
+/* Signal handler is supported. Use it by default. */
 #ifndef WASM_RT_MEMCHECK_SIGNAL_HANDLER
 #define WASM_RT_MEMCHECK_SIGNAL_HANDLER 1
 #endif
 
-#if WASM_RT_MEMCHECK_SIGNAL_HANDLER
-#define WASM_RT_MEMCHECK_SIGNAL_HANDLER_POSIX 1
+#else
+#define WASM_RT_SIGNAL_RECOVERY_SUPPORTED 0
+
+/* Signal handler is not supported. */
+#ifndef WASM_RT_MEMCHECK_SIGNAL_HANDLER
+#define WASM_RT_MEMCHECK_SIGNAL_HANDLER 0
 #endif
 
-#else
+#endif
 
+#if WASM_RT_MEMCHECK_SIGNAL_HANDLER && \
+    (!WASM_RT_SKIP_SIGNAL_RECOVERY && !WASM_RT_SIGNAL_RECOVERY_SUPPORTED)
 /* The signal handler is not supported, error out if the user was trying to
  * enable it. */
-#if WASM_RT_MEMCHECK_SIGNAL_HANDLER
 #error "Signal handler is not supported for this OS/Architecture!"
 #endif
 
-#define WASM_RT_MEMCHECK_SIGNAL_HANDLER 0
-#define WASM_RT_MEMCHECK_SIGNAL_HANDLER_POSIX 0
+#ifndef WASM_RT_USE_STACK_DEPTH_COUNT
+/* The signal handler on POSIX can detect call stack overflows. On windows, or
+ * platforms without a signal handler, we use stack depth counting. */
+#if WASM_RT_MEMCHECK_SIGNAL_HANDLER && !defined(_WIN32)
+#define WASM_RT_USE_STACK_DEPTH_COUNT 0
+#else
+#define WASM_RT_USE_STACK_DEPTH_COUNT 1
+#endif
+#endif
 
+#if WASM_RT_USE_STACK_DEPTH_COUNT
 /**
- * When the signal handler is not used, stack depth is limited explicitly.
- * The maximum stack depth before trapping can be configured by defining
- * this symbol before including wasm-rt when building the generated c files,
- * for example:
+ * When the signal handler cannot be used to detect stack overflows, stack depth
+ * is limited explicitly. The maximum stack depth before trapping can be
+ * configured by defining this symbol before including wasm-rt when building the
+ * generated c files, for example:
  *
  * ```
  *   cc -c -DWASM_RT_MAX_CALL_STACK_DEPTH=100 my_module.c -o my_module.o
@@ -101,7 +126,7 @@ extern uint32_t wasm_rt_call_stack_depth;
 #define WASM_RT_NO_RETURN __attribute__((noreturn))
 #endif
 
-#if defined(__APPLE__) && WASM_RT_MEMCHECK_SIGNAL_HANDLER_POSIX
+#if defined(__APPLE__) && WASM_RT_MEMCHECK_SIGNAL_HANDLER
 #define WASM_RT_MERGED_OOB_AND_EXHAUSTION_TRAPS 1
 #else
 #define WASM_RT_MERGED_OOB_AND_EXHAUSTION_TRAPS 0
@@ -130,27 +155,40 @@ typedef enum {
   WASM_RT_I64,
   WASM_RT_F32,
   WASM_RT_F64,
+  WASM_RT_FUNCREF,
+  WASM_RT_EXTERNREF,
 } wasm_rt_type_t;
 
 /**
- * A function type for all `funcref` functions in a Table. All functions are
- * stored in this canonical form, but must be cast to their proper signature to
- * call.
+ * A generic function pointer type, both for Wasm functions (`code`)
+ * and host functions (`hostcode`). All function pointers are stored
+ * in this canonical form, but must be cast to their proper signature
+ * to call.
  */
-typedef void (*wasm_rt_funcref_t)(void);
+typedef void (*wasm_rt_function_ptr_t)(void);
 
-/** A single element of a Table. */
+/** A function instance (the runtime representation of a function).
+ * These can be stored in tables of type funcref, or used as values. */
 typedef struct {
   /** The index as returned from `wasm_rt_register_func_type`. */
   uint32_t func_type;
   /** The function. The embedder must know the actual C signature of the
    * function and cast to it before calling. */
-  wasm_rt_funcref_t func;
+  wasm_rt_function_ptr_t func;
   /** A function instance is a closure of the function over an instance
    * of the originating module. The module_instance element will be passed into
    * the function at runtime. */
   void* module_instance;
-} wasm_rt_elem_t;
+} wasm_rt_funcref_t;
+
+/** Default (null) value of a funcref */
+static const wasm_rt_funcref_t wasm_rt_funcref_null_value = {0, NULL, NULL};
+
+/** The type of an external reference (opaque to WebAssembly). */
+typedef void* wasm_rt_externref_t;
+
+/** Default (null) value of an externref */
+static const wasm_rt_externref_t wasm_rt_externref_null_value = NULL;
 
 /** A Memory object. */
 typedef struct {
@@ -163,16 +201,27 @@ typedef struct {
   uint32_t size;
 } wasm_rt_memory_t;
 
-/** A Table object. */
+/** A Table of type funcref. */
 typedef struct {
   /** The table element data, with an element count of `size`. */
-  wasm_rt_elem_t* data;
+  wasm_rt_funcref_t* data;
   /** The maximum element count of this Table object. If there is no maximum,
    * `max_size` is 0xffffffffu (i.e. UINT32_MAX). */
   uint32_t max_size;
   /** The current element count of the table. */
   uint32_t size;
-} wasm_rt_table_t;
+} wasm_rt_funcref_table_t;
+
+/** A Table of type externref. */
+typedef struct {
+  /** The table element data, with an element count of `size`. */
+  wasm_rt_externref_t* data;
+  /** The maximum element count of this Table object. If there is no maximum,
+   * `max_size` is 0xffffffffu (i.e. UINT32_MAX). */
+  uint32_t max_size;
+  /** The current element count of the table. */
+  uint32_t size;
+} wasm_rt_externref_table_t;
 
 /** Initialize the runtime. */
 void wasm_rt_init(void);
@@ -263,7 +312,7 @@ uint32_t wasm_rt_exception_size(void);
  */
 void* wasm_rt_exception(void);
 
-#if WASM_RT_MEMCHECK_SIGNAL_HANDLER_POSIX
+#if WASM_RT_MEMCHECK_SIGNAL_HANDLER && !defined(_WIN32)
 #define WASM_RT_SETJMP(buf) sigsetjmp(buf, 1)
 #else
 #define WASM_RT_SETJMP(buf) setjmp(buf)
@@ -308,23 +357,50 @@ uint32_t wasm_rt_grow_memory(wasm_rt_memory_t*, uint32_t pages);
 void wasm_rt_free_memory(wasm_rt_memory_t*);
 
 /**
- * Initialize a Table object with an element count of `elements` and a maximum
- * page size of `max_elements`.
+ * Initialize a funcref Table object with an element count of `elements` and a
+ * maximum size of `max_elements`.
  *
  *  ```
- *    wasm_rt_table_t my_table;
- *    // 5 elemnets and a maximum of 10 elements.
- *    wasm_rt_allocate_table(&my_table, 5, 10);
+ *    wasm_rt_funcref_table_t my_table;
+ *    // 5 elements and a maximum of 10 elements.
+ *    wasm_rt_allocate_funcref_table(&my_table, 5, 10);
  *  ```
  */
-void wasm_rt_allocate_table(wasm_rt_table_t*,
-                            uint32_t elements,
-                            uint32_t max_elements);
+void wasm_rt_allocate_funcref_table(wasm_rt_funcref_table_t*,
+                                    uint32_t elements,
+                                    uint32_t max_elements);
 
 /**
- * Free a Table object.
+ * Free a funcref Table object.
  */
-void wasm_rt_free_table(wasm_rt_table_t*);
+void wasm_rt_free_funcref_table(wasm_rt_funcref_table_t*);
+
+/**
+ * Initialize an externref Table object with an element count
+ * of `elements` and a maximum size of `max_elements`.
+ * Usage as per wasm_rt_allocate_funcref_table.
+ */
+void wasm_rt_allocate_externref_table(wasm_rt_externref_table_t*,
+                                      uint32_t elements,
+                                      uint32_t max_elements);
+
+/**
+ * Free an externref Table object.
+ */
+void wasm_rt_free_externref_table(wasm_rt_externref_table_t*);
+
+/**
+ * Grow a Table object by `delta` elements (giving the new elements the value
+ * `init`), and return the previous element count. If this new element count is
+ * greater than the maximum element count, the grow fails and 0xffffffffu
+ * (UINT32_MAX) is returned instead.
+ */
+uint32_t wasm_rt_grow_funcref_table(wasm_rt_funcref_table_t*,
+                                    uint32_t delta,
+                                    wasm_rt_funcref_t init);
+uint32_t wasm_rt_grow_externref_table(wasm_rt_externref_table_t*,
+                                      uint32_t delta,
+                                      wasm_rt_externref_t init);
 
 #ifdef __cplusplus
 }
